//------------------------------------------------------------------------------
// Universitatea Transilvania din Brasov
// Departamentul de Electronica si Calculatoare
// Proiect     : Laborator HDL
// Modul       : mem_crop_test
// Autor       : Dan NICULA (DN)
// Data        : Oct. 1, 2019
//------------------------------------------------------------------------------
// Descriere   : Test pentru lucrul memoriilor cu fisiere
//------------------------------------------------------------------------------
// Modificari  :
// Oct. 1, 2019 (DN): Initial 
//------------------------------------------------------------------------------

module mem_crop_test;
wire      clk               ;
wire      rst_n             ;
      
integer   file_id           ; // handler fisier
  
integer   idx               ;
integer   row               ;
integer   col               ;
    
integer   status            ; // stare operatie cu fisiere

localparam row_crop = 'd26  ;
localparam col_crop = 'd122 ;

reg             ce_1        ;
reg             we_1        ;
reg   [14  : 0] addr_1      ;
reg   [7   : 0] wr_data_1   ;
reg             be_1        ;
wire  [7   : 0] rd_data_1   ;

reg             ce_2        ;
reg             we_2        ;
reg   [9   : 0] addr_2      ;
reg   [7   : 0] wr_data_2   ;
reg             be_2        ;
wire  [7   : 0] rd_data_2   ;

reg             ce_3        ;
reg             we_3        ;
reg   [9   : 0] addr_3      ;
reg   [7   : 0] wr_data_3   ;
reg             be_3        ;
wire  [7   : 0] rd_data_3   ;

ck_rst_tb #(
.CK_SEMIPERIOD ('d10)
) i_ck_rst_tb (
.clk    (clk   ),
.rst_n  (rst_n )
);

initial begin

  // initializeaza memoria sursa
  file_id   = $fopen("input_data/mem_map_256x128.raw", "rb");
  if (file_id == 0) begin
    $display("Nu se poate deschide fisierul input_data/mem_map_256x128.raw");
    $stop;
  end
  for (idx=0; idx<(256*128); idx=idx+1)
    status = $fread(i_mem_1rw_256x128x8.mem[idx], file_id); 
  $fclose(file_id);
 
  @(negedge rst_n);
  @(posedge rst_n);
  @(posedge clk);

  // muta datele de la sursa la destinatie
  for (row=0; row<32; row=row+1)
    for (col=0; col<32; col=col+1)
      i_mem_1rw_32x32x8.mem[row*32+col] = 
      i_mem_1rw_256x128x8.mem[(row+row_crop)*256+(col+col_crop)];
 
  // scrie memoria destinatie in fisier binar
  file_id   = $fopen("output_data/mem_map_32x32.raw", "wb");
  if (file_id == 0) begin
    $display("Nu se poate deschide fisier output_data/mem_map_32x32.raw");
    $stop;
  end
  for (idx=0; idx<(32*32); idx=idx+1)
    $fwrite(file_id, "%c", i_mem_1rw_32x32x8.mem[idx]);
  $fclose(file_id);

  // scrie memoria destinatie in fisier text, hexa
  file_id   = $fopen("output_data/mem_map_32x32.hex", "w");
  if (file_id == 0) begin
    $display("Nu se poate deschide fisier output_data/mem_map_32x32.hex");
    $stop;
  end
  for (idx=0; idx<(32*32); idx=idx+1)
    $fwrite(file_id,"%h \n", i_mem_1rw_32x32x8.mem[idx]);
  $fclose(file_id);

  // scrie memoria destinatie in fisier text, binar
  file_id   = $fopen("output_data/mem_map_32x32.bin", "w");
  if (file_id == 0) begin
    $display("Nu se poate deschide fisier output_data/mem_map_32x32.bin");
    $stop;
  end
  for (idx=0; idx<(32*32); idx=idx+1)
    $fwrite(file_id,"%b \n", i_mem_1rw_32x32x8.mem[idx]);
  $fclose(file_id);
  
  // initializeaza memoria cu $readmemh sau $readmemb
//  $readmemh("output_data/mem_map_32x32.hex", i_mem_1rw_32x32x8_rd.mem);
  $readmemb("output_data/mem_map_32x32.bin", i_mem_1rw_32x32x8_rd.mem);
  
  // verifica continutul memoriilor
  for (idx=0; idx<(32*32); idx=idx+1)
    if (i_mem_1rw_32x32x8_rd.mem[idx] != i_mem_1rw_32x32x8.mem[idx]) begin
      $display("EROARE copiere date in memorie");
      $stop;
    end
    
  $display ("%M %0t INFO: Final simulare.", $time);
  $stop;
end

mem_1rw  #(
.ADDR_WIDTH (15     )  ,
.MEM_DEPTH  (1 << 15)  ,
.WORD_BYTES (1      )    // latimea datelor 8 biti
) i_mem_1rw_256x128x8(
.clk       (clk       ),
.ce        (ce_1      ), 
.we        (we_1      ), 
.addr      (addr_1    ), 
.wr_data   (wr_data_1 ), 
.be        (be_1      ), 
.rd_data   (rd_data_1 )  
);

mem_1rw  #(
.ADDR_WIDTH (10      ),
.MEM_DEPTH  (1 << 10 ),
.WORD_BYTES (1       )  // latimea datelor  8 biti
) i_mem_1rw_32x32x8(
.clk       (clk       ),
.ce        (ce_2      ), 
.we        (we_2      ), 
.addr      (addr_2    ), 
.wr_data   (wr_data_2 ), 
.be        (be_2      ), 
.rd_data   (rd_data_2 )  
);

mem_1rw  #(
.ADDR_WIDTH (10      ),
.MEM_DEPTH  (1 << 10 ),
.WORD_BYTES (1       )  // latimea datelor  8 biti
) i_mem_1rw_32x32x8_rd(
.clk       (clk       ),
.ce        (ce_3      ), 
.we        (we_3      ), 
.addr      (addr_3    ), 
.wr_data   (wr_data_3 ), 
.be        (be_3      ), 
.rd_data   (rd_data_3 )  
);

endmodule // mem_crop_test
